import socket

from cc.serializer import SocketSerializer

class GenericInstrument:
    """
    Generic instrument class.
    """
    
    def __init__(self, ip_address, port=5025):
        self._sock = None
        self.ip_address = ip_address
        self.port = port
    
    def connect(self):
        """
        Connect to the instrument.
        """
        self._sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        
        addr = (self.ip_address, self.port)
        try:
            self._sock.connect(addr)
        except socket.error as e:
            print("failed to connect: ", addr)
            print(e)
        
        return self._sock

    def close(self):
        """
        Close the connection.
        """
        self._sock.close()
    
    def command(self, cmd: str):
        """
        Send a command to the instrument.
        
        Args:
            cmd: the command to send
        """
        ser = SocketSerializer(self._sock, buffered_transmit=True)
        ser.transmit(cmd.encode())
    
    def query(self, cmd: str):
        """
        Send a query to the instrument.

        Args:
            cmd: the command to send
        
        Returns:
            the response from the instrument
        """
        ser = SocketSerializer(self._sock, buffered_transmit=True)
        ser.transmit(cmd.encode())
        data = ser.receive()
        return data.decode()

    def get_idn(self):
        """
        Get the instrument identification number

        Returns:
            the instrument identification
        """
        return self.query("*IDN?")

    def get_instrument_identification(self):
        """
        Get the instrument identification number. This is an alias for `get_idn()`.

        Returns:
            the instrument identification
        """
        return self.get_idn()
    




from .generic_instrument import GenericInstrument

class SiglentSPD3303X(GenericInstrument):
    """
    Siglent SPD3303X Programmable DC Power Supply

    The Siglent SPD3303X Programmable DC Power Supply is a versatile power supply with the capabilities of generating constant voltage and constant current. 

    https://siglentna.com/power-supplies/spd3303x-spd3303x-e-series-programmable-dc-power-supply/    
    """
    
    class Channel:
        """
        The channel of the output.
        """
        CH1 = "CH1"
        CH2 = "CH2"
    
    def get_current(self, channel: Channel):
        """
        Get the current reading of the output.

        Args:
            channel: the target channel

        Returns:
            the current reading of the output in amperes (A)
        """
        return float(self.query("MEASURE:CURRENT? {ch}".format(ch=channel)))
    
    def get_voltage(self, channel: Channel):
        """
        Get the voltage reading of the output.
        
        Args:
            channel: the target channel
        
        Returns:
            the voltage reading of the output in volts (V)
        """
        return float(self.query("MEASURE:VOLTAGE? {ch}".format(ch=channel)))
    
    def get_power(self, channel: Channel):
        """
        Get the power reading of the output.

        Args:
            channel: the target channel
        
        Returns:
            the power reading of the output in watts (W)
        """
        return float(self.query("MEASURE:POWER? {ch}".format(ch=channel)))
    
    def set_current(self, value: float, channel: Channel):
        """
        Set the current limit of the output.
        
        Args:
            value: the target current limit value in amperes (A)
            channel: the target channel
        """
        self.command("{ch}:CURRENT {val}".format(ch=channel, val=value))
        
    def set_voltage(self, value: float, channel: Channel):
        """
        Set the voltage of the output.

        Args:
            value: the target voltage value in volts (V)
            channel: the target channel
        """
        self.command("{ch}:VOLTAGE {val}".format(ch=channel, val=value))

    def disable_output(self, channel: Channel):
        """
        Disable the output.
        
        Args:
            channel: the target channel
        """
        self.command("OUTPUT {ch},OFF".format(ch=channel))

    def enable_output(self, channel: Channel):
        """
        Enable the output.

        Args:
            channel: the target channel
        """
        self.command("OUTPUT {ch},ON".format(ch=channel))


from setuptools import setup, find_namespace_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="cc.scpi",
    version="2024.8.30",
    author="Uncertainty.",
    author_email="t_k_233@outlook.email",
    description="Wrapper of SCPI language for controlling various lab instruments.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/uncertainty-cc/SCPI-Python",
    project_urls={
        
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        "cc-serializer",
    ],
    package_dir={"": "src/"},
    packages=find_namespace_packages(where="src/", include=["cc.scpi"]),
    python_requires=">=3.8",
)




from .generic_instrument import GenericInstrument

class Keysight33600A(GenericInstrument):
    """
    Keysight 33600A Series Waveform Generator
    
    The Keysight 33600A Series Waveform Generator is a versatile signal generator with the capabilities of generating sine, square, ramp, pulse, and arbitrary waveforms. It can be used to test analog designs, such as filters, amplifiers, etc.
    
    https://www.keysight.com/fr/en/assets/7018-04123/data-sheets-archived/5991-3272.pdf
    """
    
    class Function:
        """
        The function of the output waveform.
        """
        SQUARE = "SQUARE"

    class Channel:
        """
        The channel of the output waveform.
        """
        CH1 = "1"
        CH2 = "2"
    
    def set_function(self, function: Function, channel: Channel):
        """
        Set function of both channels

        Args:
            function: the target function
            channel: the target channel
        """
        self.command("SOURCE{ch}:FUNCTION {func}".format(ch=channel, func=function))
    
    def set_frequency(self, value: float, channel: Channel):
        """
        Set frequency of both channels
        
        Args:
            value: the target frequency value in hertz (Hz). The value must be within [100, 600000000] Hz.
            channel: the target channel
        """
        self.command("SOURCE{ch}:FREQUENCY {val}".format(ch=channel, val=value))
    
    def set_high_voltage(self, value: float, channel: Channel):
        """
        Set the voltage level of the high state of the output waveform.

        Args:
            value: the target voltage value in volts (V).
            channel: the target channel
        """
        self.command("SOURCE{ch}:VOLTAGE:HIGH {val}".format(ch=channel, val=value))
    
    def set_low_voltage(self, value: float, channel: Channel):
        """
        Set the voltage level of the low state of the output waveform.

        Args:
            value: the target voltage value in volts (V).
            channel: the target channel
        """
        self.command("SOURCE{ch}:VOLTAGE:LOW {val}".format(ch=channel, val=value))
    
    def set_output_load(self, value: float, channel: Channel):
        """
        Set the output load impedance.

        Args:
            value: the target load value in ohms (Ω). The value must be within [1, 1000000] Ω.
            channel: the target channel
        """
        if value == float("inf"):
            value = "INFINITY"
        self.command("OUTPUT{ch}:LOAD {val}".format(ch=channel, val=value))

    def disable_output(self, channel: Channel):
        """
        Disable the output of the specified channel.

        Args:
            channel: the target channel
        """
        self.command("OUTPUT{ch} OFF".format(ch=channel))

    def enable_output(self, channel: Channel):
        """
        Enable the output of the specified channel.

        Args:
            channel: the target channel
        """
        self.command("OUTPUT{ch} ON".format(ch=channel))


class Keysight81134A(GenericInstrument):
    """
    Keysight 81134A Pulse Pattern Generator, Dual-Channel

    The Keysight 81134A is a dual-channel, 3.35 GHz Pulse Pattern Generator. It is used to test digital designs from DC to 3.35 GHz. It can be used to test digital interfaces, such as HDMI, MIPI, DisplayPort, SATA, USB, DDR, etc.

    https://www.keysight.com/fr/en/assets/7018-01085/data-sheets/5988-5549.pdf
    """

    class Channel:
        """
        The channel of the output waveform.
        """
        CH1 = "1"
        CH2 = "2"
    
    class Function:
        """
        The function of the output waveform.
        """
        SQUARE = "SQUARE"
        DATA = "DATA"
    
    def set_function(self, function: Function, channel: Channel):
        """
        Set function of both channels

        Args:
            function: the target function
            channel: the target channel
        """
        self.command(":SOURCE:FUNCTION:MODE{ch} {func}".format(ch=channel, func=function))
    
    def set_frequency(self, value: int):
        """
        Set frequency of both channels

        Args:
            value: the target frequency value in Hz. The value must be within [15000000, 3350000000] Hz.
        """
        assert 15000000. <= value and value <= 3350000000.
        self.command(":FREQUENCY {val}".format(val=value))
    
    def set_frequency_divisor(self, value: int, channel: Channel):
        """
        Set the frequency divider of both channels
        
        Args:
            value: the target frequency divider value. The value must be one of 1, 2, 4, 8, 16, 32, 64, 128
        """
        assert value in [1, 2, 4, 8, 16, 32, 64, 128], "Invalid frequency divisor"
        self.command(":OUTPUT{ch}:DIVIDER {val}".format(ch=channel, val=value))
    
    def set_voltage_high(self, value: float, channel: Channel):
        """
        Set the voltage level of the high state of the output waveform.
        
        Args:
            value: the target voltage value in volts (V). The value must be within [0, 2] V.
            channel: the target channel
        """
        assert 0. <= value and value <= 2.
        self.command(":SOURCE:VOLTAGE{ch}:HIGH {val}V".format(ch=channel, val=value))

    def set_voltage_low(self, value: float, channel: Channel):
        """
        Set the voltage level of the high state of the output waveform.
        
        Args:
            value: the target voltage value in volts (V). The value must be within [0, 2] V.
            channel: the target channel
        """
        assert 0. <= value and value <= 2.
        self.command(":SOURCE:VOLTAGE{ch}:LOW {val}V".format(ch=channel, val=value))
    
    def disable_output(self, channel: Channel):
        """
        Disable the output of both phases of the specified channel.

        Args:
            channel: the target channel
        """
        self.command(":OUTPUT{ch} OFF".format(ch=channel))

    def disable_output_p(self, channel: Channel):
        """
        Disable the output of the positive phase of the specified channel.

        Args:
            channel: the target channel
        """
        self.command(":OUTPUT{ch}:POS OFF".format(ch=channel))

    def disable_output_n(self, channel: Channel):
        """
        Disable the output of the negative phase of the specified channel.

        Args:
            channel: the target channel
        """
        self.command(":OUTPUT{ch}:NEG OFF".format(ch=channel))

    def enable_output(self, channel: Channel):
        """
        Enable the output of both phases of the specified channel.

        Args:
            channel: the target channel
        """
        self.command(":OUTPUT{ch} ON".format(ch=channel))

    def enable_output_p(self, channel: Channel):
        """
        Enable the output of the positive phase of the specified channel.

        Args:
            channel: the target channel
        """
        self.command(":OUTPUT{ch}:POS ON".format(ch=channel))

    def enable_output_n(self, channel: Channel):
        """
        Enable the output of the negative phase of the specified channel.

        Args:
            channel: the target channel
        """
        self.command(":OUTPUT{ch}:NEG ON".format(ch=channel))
